@@ -207,63 +207,84 @@ fit_sq_loss_lasso <- function(x, y, trt, n.trts, wts, family, match.id, intercep
     }
     list.dots$nfolds <- nfolds
 
-    ## Establish foldid for cv.glmnet()
-    ## if match.id was supplied, foldid will be structured around the clusters
-    if (!is.null(match.id))
-    {
-        if ("foldid" %in% dot.names)
-        {
-            warning("User-supplied foldid will be ignored since match.id was detected.
-                    Folds will be randomly assigned to clusters according to match.id.")
-        }
-        # Assign a fold ID for each cluster level
-        df.folds <- data.frame(match.id = sample(levels(match.id)),
-                               fold.id = 1:length(levels(match.id)) %% nfolds)
-        # Obtain vector of fold IDs with respect to the data
-        foldid <- sapply(match.id, function(z) {df.folds[which(z == df.folds$match.id),"fold.id"]}) +1
-    } else
-    {
-        if ("foldid" %in% dot.names)
-        {
-            foldid <- list.dots$foldid
-        } else
-        {
-            foldid <- sample(rep(seq(nfolds), length = nrow(x)))
-        }
-    }
-    list.dots$foldid <- foldid
+    nsel <- 0
+    ct   <- 0
+    ntry <- 4
 
-    # fit a model with a lasso
-    # penalty and desired loss
-    model <- do.call(cv.glmnet, c(list(x = x, y = y, weights = wts, family = family), list.dots))
-
-    # this is needed for OWL losses, as glmnet
-    # no longer allows constant columns (ie an intercept)
-    # to have estimated coefficients
-    if (intercept)
+    while(nsel == 0 & ct <= ntry)
     {
-        if (family != "multinomial")
+        ct <- ct + 1
+        ## Establish foldid for cv.glmnet()
+        ## if match.id was supplied, foldid will be structured around the clusters
+        if (!is.null(match.id))
         {
-            model$glmnet.fit$beta[1,] <- unname(model$glmnet.fit$a0)
-            model$glmnet.fit$a0       <- rep(0, length(model$glmnet.fit$a0))
-        } else
-        {
-            for (cl in 1:nrow(model$glmnet.fit$a0))
+            if ("foldid" %in% dot.names)
             {
-                model$glmnet.fit$beta[[cl]][1,] <- unname(model$glmnet.fit$a0[cl,])
-                model$glmnet.fit$a0[cl,]        <- rep(0, length(model$glmnet.fit$a0[cl,]))
+                warning("User-supplied foldid will be ignored since match.id was detected.
+                        Folds will be randomly assigned to clusters according to match.id.")
+            }
+            # Assign a fold ID for each cluster level
+            df.folds <- data.frame(match.id = sample(levels(match.id)),
+                                   fold.id = 1:length(levels(match.id)) %% nfolds)
+            # Obtain vector of fold IDs with respect to the data
+            foldid <- sapply(match.id, function(z) {df.folds[which(z == df.folds$match.id),"fold.id"]}) + 1
+        } else
+        {
+            if ("foldid" %in% dot.names)
+            {
+                foldid <- list.dots$foldid
+            } else
+            {
+                foldid <- sample(rep(seq(nfolds), length = nrow(x)))
             }
         }
+        list.dots$foldid <- foldid
+
+        # fit a model with a lasso
+        # penalty and desired loss
+        model <- do.call(cv.glmnet, c(list(x = x, y = y, weights = wts, family = family), list.dots))
+
+        # this is needed for OWL losses, as glmnet
+        # no longer allows constant columns (ie an intercept)
+        # to have estimated coefficients
+        if (intercept)
+        {
+            if (family != "multinomial")
+            {
+                model$glmnet.fit$beta[1,] <- unname(model$glmnet.fit$a0)
+                model$glmnet.fit$a0       <- rep(0, length(model$glmnet.fit$a0))
+            } else
+            {
+                for (cl in 1:nrow(model$glmnet.fit$a0))
+                {
+                    model$glmnet.fit$beta[[cl]][1,] <- unname(model$glmnet.fit$a0[cl,])
+                    model$glmnet.fit$a0[cl,]        <- rep(0, length(model$glmnet.fit$a0[cl,]))
+                }
+            }
+        }
+
+        coefs <- get.coef.func("fit_sq_loss_lasso")(model)
+
+        if (is.list(coefs))
+        {
+            nsel  <- sum(sapply(coefs, function(cfs) sum(cfs != 0))) - (n.trts - 1)
+        } else
+        {
+            nsel  <- sum(coefs != 0) - (n.trts - 1)
+        }
     }
 
+
+
     # Return fitted model and extraction methods
     list(predict      = get.pred.func("fit_sq_loss_lasso", model),
          model        = model,
-         coefficients = get.coef.func("fit_sq_loss_lasso")(model))
+         coefficients = coefs)
 }
 
 
 fit_logistic_loss_lasso <- fit_sq_loss_lasso
+fit_poisson_loss_lasso  <- fit_sq_loss_lasso
 
 #' @import survival
 fit_cox_loss_lasso <- function(x, y, trt, n.trts, wts, family, match.id, ...)
@@ -299,20 +320,27 @@ fit_cox_loss_lasso <- function(x, y, trt, n.trts, wts, family, match.id, ...)
     }
     list.dots$nfolds <- nfolds
 
-    ## Establish foldid for cv.glmnet()
-    ## if match.id was supplied, foldid will be structured around the clusters
-    if (!is.null(match.id))
+    nsel <- 0
+    ct   <- 0
+    ntry <- 4
+
+    while(nsel == 0 & ct <= ntry)
     {
-        if ("foldid" %in% dot.names)
+        ct <- ct + 1
+        ## Establish foldid for cv.glmnet()
+        ## if match.id was supplied, foldid will be structured around the clusters
+        if (!is.null(match.id))
         {
-            warning("User-supplied foldid will be ignored since match.id was detected.
-                    Folds will be randomly assigned to clusters according to match.id.")
-        }
-        # Assign a fold ID for each cluster level
-        df.folds <- data.frame(match.id = sample(levels(match.id)),
-                               fold.id = 1:length(levels(match.id)) %% nfolds)
-        # Obtain vector of fold IDs with respect to the data
-        foldid <- sapply(match.id, function(z) {df.folds[which(z == df.folds$match.id),"fold.id"]}) +1
+            if ("foldid" %in% dot.names)
+            {
+                warning("User-supplied foldid will be ignored since match.id was detected.
+                        Folds will be randomly assigned to clusters according to match.id.")
+            }
+            # Assign a fold ID for each cluster level
+            df.folds <- data.frame(match.id = sample(levels(match.id)),
+                                   fold.id = 1:length(levels(match.id)) %% nfolds)
+            # Obtain vector of fold IDs with respect to the data
+            foldid <- sapply(match.id, function(z) {df.folds[which(z == df.folds$match.id),"fold.id"]}) +1
         } else
         {
             if ("foldid" %in% dot.names)
@@ -323,16 +351,29 @@ fit_cox_loss_lasso <- function(x, y, trt, n.trts, wts, family, match.id, ...)
                 foldid <- sample(rep(seq(nfolds), length = nrow(x)))
             }
         }
-    list.dots$foldid <- foldid
+        list.dots$foldid <- foldid
+
+        # fit a model with a lasso
+        # penalty and desired loss
+        model <- do.call(cv.glmnet, c(list(x = x, y = y, weights = wts, family = "cox"), list.dots))
+
+        coefs <- get.coef.func("fit_cox_loss_lasso")(model)
+
+        if (is.list(coefs))
+        {
+            nsel  <- sum(sapply(coefs, function(cfs) sum(cfs != 0))) - (n.trts - 1)
+        } else
+        {
+            nsel  <- sum(coefs != 0) - (n.trts - 1)
+        }
+    }
+
 
-    # fit a model with a lasso
-    # penalty and desired loss
-    model <- do.call(cv.glmnet, c(list(x = x, y = y, weights = wts, family = "cox"), list.dots))
 
     # Return fitted model and extraction methods
     list(predict      = get.pred.func("fit_cox_loss_lasso", model),
          model        = model,
-         coefficients = get.coef.func("fit_cox_loss_lasso")(model))
+         coefficients = coefs)
 }
 
 
@@ -393,23 +434,23 @@ fit_sq_loss_lasso_gam <- function(x, y, trt, n.trts, wts, family, match.id, inte
         if ("foldid" %in% dot.names)
         {
             warning("User-supplied foldid will be ignored since match.id was detected.
-                    Folds will be randomly assigned to clusters according to match.id.")
+                     Folds will be randomly assigned to clusters according to match.id.")
         }
         # Assign a fold ID for each cluster level
         df.folds <- data.frame(match.id = sample(levels(match.id)),
                                fold.id = 1:length(levels(match.id)) %% nfolds)
         # Obtain vector of fold IDs with respect to the data
-        foldid <- sapply(match.id, function(z) {df.folds[which(z == df.folds$match.id),"fold.id"]}) +1
+        foldid <- sapply(match.id, function(z) {df.folds[which(z == df.folds$match.id),"fold.id"]}) + 1
+    } else
+    {
+        if ("foldid" %in% dot.names)
+        {
+            foldid <- list.dots$foldid
         } else
         {
-            if ("foldid" %in% dot.names)
-            {
-                foldid <- list.dots$foldid
-            } else
-            {
-                foldid <- sample(rep(seq(nfolds), length = nrow(x)))
-            }
+            foldid <- sample(rep(seq(nfolds), length = nrow(x)))
         }
+    }
     list.dots$foldid <- foldid
 
     glmnet.argnames <- union(names(formals(cv.glmnet)), names(formals(glmnet)))
@@ -484,6 +525,10 @@ fit_sq_loss_lasso_gam <- function(x, y, trt, n.trts, wts, family, match.id, inte
         {
             family.func <- binomial()
             y <- as.integer(y)
+        } else if (family == "poisson")
+        {
+            family.func <- poisson()
+            y <- as.integer(y)
         }
     }
     gam.formula <- as.formula(paste("y ~", rhs.formula))
@@ -517,6 +562,7 @@ fit_sq_loss_lasso_gam <- function(x, y, trt, n.trts, wts, family, match.id, inte
 
 fit_logistic_loss_lasso_gam <- fit_sq_loss_lasso_gam
 fit_cox_loss_lasso_gam      <- fit_sq_loss_lasso_gam
+fit_poisson_loss_lasso_gam  <- fit_sq_loss_lasso_gam
 
 
 
@@ -596,6 +642,10 @@ fit_sq_loss_gam <- function(x, y, trt, n.trts, wts, family, match.id, ...)
         {
             family.func <- binomial()
             y <- as.integer(y)
+        } else if (family == "poisson")
+        {
+            family.func <- poisson()
+            y <- as.integer(y)
         }
     }
     gam.formula <- as.formula(paste("y ~", rhs.formula))
@@ -629,6 +679,7 @@ fit_sq_loss_gam <- function(x, y, trt, n.trts, wts, family, match.id, ...)
 }
 
 fit_logistic_loss_gam <- fit_sq_loss_gam
+fit_poisson_loss_gam  <- fit_sq_loss_gam
 fit_cox_loss_gam      <- fit_sq_loss_gam
 
 
@@ -691,7 +742,7 @@ fit_sq_loss_gbm <- function(x, y, trt, n.trts, wts, family, match.id, ...)
     # penalty and desired loss
     model <- do.call(gbm, c(list(formula.gbm, data = df,
                                  weights = wts,
-                                 distribution = "gaussian"),
+                                 distribution = family),
                             list.dots))
 
     best.iter <- gbm.perf(model, method = "cv")
@@ -704,74 +755,77 @@ fit_sq_loss_gbm <- function(x, y, trt, n.trts, wts, family, match.id, ...)
          coefficients = get.coef.func("fit_sq_loss_gbm")(model))
 }
 
+fit_poisson_loss_gbm <- fit_sq_loss_gbm
 
-fit_abs_loss_gbm <- function(x, y, trt, n.trts, wts, family, match.id, ...)
-{
-    # this function must return a fitted model
-    # in addition to a function which takes in
-    # a design matrix and outputs estimated benefit scores
-
-    ###################################################################
-    ##
-    ## IMPORTANT NOTE: the name of this function *must*
-    ##                 begin with "fit_" and end with
-    ##                 the text string to associated with
-    ##                 this function in the options for the
-    ##                 'loss' argument of the fit.subgrp()
-    ##                 function
-    ##
-    ###################################################################
-
-    list.dots <- list(...)
-
-    dot.names <- names(list.dots)
-    if ("cv.folds" %in% dot.names)
-    {
-        cv.folds <- list.dots["cv.folds"]
-        if (cv.folds < 2)
-        {
-            cv.folds <- 2L
-            list.dots$cv.folds <- cv.folds
-            warning("cv.folds must be at least 2, setting cv.folds to 2.")
-        }
-
-    } else
-    {
-        list.dots$cv.folds <- 5L
-    }
-
-    if (!is.null(match.id))
-    {
-        warning("Matched groups are not guaranteed to remain matched in the cross-validation procedure using GBM models.")
-    }
-
-    if ("offset" %in% dot.names)
-    {
-        df <- data.frame(y = y, x, offset = list.dots$offset)
-        list.dots$offset <- NULL
-    } else
-    {
-        df <- data.frame(y = y, x, offset = rep(0,NROW(x)))
-    }
-
-    formula.gbm <- as.formula("y ~ . - 1 + offset(offset)")
-
-    # fit a model with a lasso
-    # penalty and desired loss
-    model <- do.call(gbm, c(list(formula.gbm, data = df,
-                                 weights = wts,
-                                 distribution = "laplace"),
-                            list.dots))
-
-    best.iter <- gbm.perf(model, method = "cv")
-
-    vnames <- colnames(df)[-1]
-
-    # Return fitted model and extraction methods
-    list(predict      = get.pred.func("fit_abs_loss_gbm", model),
-         model        = model,
-         coefficients = get.coef.func("fit_abs_loss_gbm")(model))
-}
+#
+#
+# fit_abs_loss_gbm <- function(x, y, trt, n.trts, wts, family, match.id, ...)
+# {
+#     # this function must return a fitted model
+#     # in addition to a function which takes in
+#     # a design matrix and outputs estimated benefit scores
+#
+#     ###################################################################
+#     ##
+#     ## IMPORTANT NOTE: the name of this function *must*
+#     ##                 begin with "fit_" and end with
+#     ##                 the text string to associated with
+#     ##                 this function in the options for the
+#     ##                 'loss' argument of the fit.subgrp()
+#     ##                 function
+#     ##
+#     ###################################################################
+#
+#     list.dots <- list(...)
+#
+#     dot.names <- names(list.dots)
+#     if ("cv.folds" %in% dot.names)
+#     {
+#         cv.folds <- list.dots["cv.folds"]
+#         if (cv.folds < 2)
+#         {
+#             cv.folds <- 2L
+#             list.dots$cv.folds <- cv.folds
+#             warning("cv.folds must be at least 2, setting cv.folds to 2.")
+#         }
+#
+#     } else
+#     {
+#         list.dots$cv.folds <- 5L
+#     }
+#
+#     if (!is.null(match.id))
+#     {
+#         warning("Matched groups are not guaranteed to remain matched in the cross-validation procedure using GBM models.")
+#     }
+#
+#     if ("offset" %in% dot.names)
+#     {
+#         df <- data.frame(y = y, x, offset = list.dots$offset)
+#         list.dots$offset <- NULL
+#     } else
+#     {
+#         df <- data.frame(y = y, x, offset = rep(0,NROW(x)))
+#     }
+#
+#     formula.gbm <- as.formula("y ~ . - 1 + offset(offset)")
+#
+#     # fit a model with a lasso
+#     # penalty and desired loss
+#     model <- do.call(gbm, c(list(formula.gbm, data = df,
+#                                  weights = wts,
+#                                  distribution = "laplace"),
+#                             list.dots))
+#
+#     best.iter <- gbm.perf(model, method = "cv")
+#
+#     vnames <- colnames(df)[-1]
+#
+#     # Return fitted model and extraction methods
+#     list(predict      = get.pred.func("fit_abs_loss_gbm", model),
+#          model        = model,
+#          coefficients = get.coef.func("fit_abs_loss_gbm")(model))
+# }
 
 
 fit_logistic_loss_gbm <- function(x, y, trt, n.trts, wts, family, match.id, ...)
@@ -843,6 +897,75 @@ fit_logistic_loss_gbm <- function(x, y, trt, n.trts, wts, family, match.id, ...)
 }
 
 
+#
+# fit_poisson_loss_gbm <- function(x, y, trt, n.trts, wts, family, match.id, ...)
+# {
+#     # this function must return a fitted model
+#     # in addition to a function which takes in
+#     # a design matrix and outputs estimated benefit scores
+#
+#     ###################################################################
+#     ##
+#     ## IMPORTANT NOTE: the name of this function *must*
+#     ##                 begin with "fit_" and end with
+#     ##                 the text string to associated with
+#     ##                 this function in the options for the
+#     ##                 'loss' argument of the fit.subgrp()
+#     ##                 function
+#     ##
+#     ###################################################################
+#
+#     list.dots <- list(...)
+#
+#     dot.names <- names(list.dots)
+#     if ("cv.folds" %in% dot.names)
+#     {
+#         cv.folds <- list.dots["cv.folds"]
+#         if (cv.folds < 2)
+#         {
+#             cv.folds <- 2L
+#             list.dots$cv.folds <- cv.folds
+#             warning("cv.folds must be at least 2, setting cv.folds to 2.")
+#         }
+#
+#     } else
+#     {
+#         list.dots$cv.folds <- 5L
+#     }
+#
+#     if (!is.null(match.id))
+#     {
+#         warning("Matched groups are not guaranteed to remain matched in the cross-validation procedure using GBM models.")
+#     }
+#
+#     if ("offset" %in% dot.names)
+#     {
+#         df <- data.frame(y = y, x, offset = list.dots$offset)
+#         list.dots$offset <- NULL
+#     } else
+#     {
+#         df <- data.frame(y = y, x, offset = rep(0,NROW(x)))
+#     }
+#
+#     formula.gbm <- as.formula("y ~ . - 1 + offset(offset)")
+#
+#     # fit a model with a lasso
+#     # penalty and desired loss
+#     model <- do.call(gbm, c(list(formula.gbm, data = df,
+#                                  weights = wts,
+#                                  distribution = "poisson"),
+#                             list.dots))
+#
+#     best.iter <- gbm.perf(model, method = "cv")
+#
+#     vnames <- colnames(df)[-1]
+#
+#     # Return fitted model and extraction methods
+#     list(predict      = get.pred.func("fit_poisson_loss_gbm", model),
+#          model        = model,
+#          coefficients = get.coef.func("fit_poisson_loss_gbm")(model))
+# }
+#
 
 fit_cox_loss_gbm <- function(x, y, trt, n.trts, wts, family, match.id, ...)
 {
